#include <iostream.h>
#include <stdlib.h>
#include "mypoint.hpp" 
#include "pointset.hpp" 
#include "segment.hpp" 
#include "cone.hpp" 

void updateConvexHull( int startindex, int endindex, int **CHminus, int **CHplus, pointset *q, pointset *r ) {
    segment *seg = new segment() ;
    int p = endindex - 1 ;
    while ( p != startindex && !seg->aboveSegment( q->get(endindex), q->get(p), q->get( CHplus[p][0] ) ) ) 
      p = CHplus[p][0] ;
    CHplus[p][1] = endindex ;
    CHplus[endindex][0] = p ;

    p = endindex - 1 ;
    while ( p != startindex && seg->aboveSegment( r->get(endindex), r->get(p), r->get( CHminus[p][0] ) ) ) 
      p = CHminus[p][0] ;
    CHminus[p][1] = endindex ;
    CHminus[endindex][0] = p ; 
}

void updateSeparators( int startindex, int endindex, int **CHminus, int **CHplus, pointset *q, pointset *r , int *tangents ){
    segment *seg = new segment() ;
    
    if ( !seg->aboveSegment( q->get(endindex), r->get(tangents[1]) , q->get(tangents[2]) ) ) {
      tangents[2] = endindex ;
      while ( !seg->aboveSegment ( q->get(endindex), r->get(tangents[1]), r->get( CHminus[tangents[1]][1] ) ) ) {
	tangents[1] = CHminus[tangents[1]][1] ;
      }
    }
    
    if ( seg->aboveSegment( r->get(endindex), q->get(tangents[0]) , r->get(tangents[3]) ) ) {
      tangents[3] = endindex ;
      while ( seg->aboveSegment ( r->get(endindex), q->get(tangents[0]), q->get( CHplus[tangents[0]][1] ) ) ) {
	tangents[0] = CHplus[tangents[0]][1] ;
      }
    }
}
  

int simplifyApproximation (pointset *invec, leda_array<int> & outvec, double eps) {
      int count = invec->size()-1 ;
      int toreturn = 0; 
      pointset *q, *r ;
      q = new pointset(count+1) ; r = new pointset(count+1) ;
      q->copyWithYTrans(invec, -eps/2) ;
      r->copyWithYTrans(invec, +eps/2) ;
  
      int  i; 
      
      segment *seg = new segment();
      
      int **CHminus = new int *[count+1];   
      for (i = 0;i<count+1;i++) CHminus[i]  = new int[2];  

      int **CHplus = new int *[count+1];   
      for (i = 0;i<count+1;i++) CHplus[i]  = new int[2];  

      CHminus[0][1] = 1 ; CHplus[0][1]  = 1 ;
      CHminus[1][0] = 0 ;
      CHplus[1][0]  = 0 ;

      int endindex = 2 , startindex = 0 ;
      
      int *tangents = new int[4] ;
      
      tangents[0] = tangents[1] = 0 ; // tangent[0] = uq, tangent[1] = ur
      tangents[2] = tangents[3] = 1 ; // tangent[2] = vq, tangent[3] = vr
      
      outvec[ toreturn++ ] =  0 ;
      
      for (; endindex <= count ; endindex++ ) {
	updateConvexHull( startindex, endindex, CHminus, CHplus, q, r ) ;

	updateSeparators( startindex, endindex, CHminus, CHplus, q, r , tangents ) ;
	if ( seg->aboveSegment( q->get(endindex), q->get( tangents[0] ), r->get( tangents[3]) ) == true && seg->aboveSegment(r->get(endindex), r->get( tangents[1] ), q->get(tangents[2]) ) == false ) {
	  continue ;
	}
	else {
          outvec[ toreturn++ ] =  endindex - 1 ;
	  startindex = endindex - 1;
	  tangents[0] = tangents[1] = startindex ;
	  tangents[2] = tangents[3] = endindex  ; 
	  CHminus[startindex][1] = endindex ; CHplus[startindex][1] = endindex ;
	  CHminus[endindex][0] = startindex ; CHplus[endindex][0] = startindex ;
	}
      }
      outvec[ toreturn++ ] =  invec->size() - 1 ; 
      return toreturn ; 
}

int simplifyUniform (pointset *invec, leda_array<int> & outvec, double eps) {
  long i, size, count ;

  cone *mycone = new cone( invec->get(0) ) ;  
//  mycone->print( ) ; 

  size = invec->size()  ; 
  count = 0; 
  outvec[count++] = 0 ; 

  for (i=1;i<size - 1 ; i++) {
    mycone->doUpdate( invec->get(i), eps ) ; 
    if ( !mycone->isContains( invec->get(i+1) ) ) {
      outvec[count++] = i ; 
      mycone->initialize( invec->get(i) ) ; 
    } 
  }
  outvec[count++] = size - 1 ; 
  return count; 
}
