#include "frechet.h"

Frechet::Frechet()
{

}

Frechet::~Frechet()
{
    RemoveAll();
}

void Frechet::RemoveAll()
{
    m_P = NULL;
    m_Q = NULL;

}

// P is the curve, Q is the segment ALWAYS... 
int Frechet::Decision(Polyline& P, Polyline& Q, DType epsilon)
{
    m_P = &P;
    m_Q = &Q;
    return(Decision(epsilon));
}

// currently, the version is only for a polygon line with a seg.
// the reason is I want it to be as fast as possible. 
int Frechet::Decision(DType epsilon)
{
    // array<point> cellsides(4); // the order of the 4 sides are counterclockwise. 
    // Each point stores the valid range. ([0,1] range). 
    // The upper range is <= 0.0 if the intersection is empty. 
    
    array<point> twosides(2); //2sides[0] is the range on Q by p0,
    // 2sides[1] is the range on Q by p1. 
    // The upper range is < 0.0 if the intersection is empty. 

    float low, lownew;
    m_epsilon = epsilon;
    
    // THis is 2D
    // setefgh((*m_Q)[0], (*m_Q)[1]);
    // 3D
    setghijkl((*m_Q)[0], (*m_Q)[1]);

    for(int i = 0; i < m_P->size()-1; i ++) {
        // since Q is only a segment,
        // for each grid cell, we only need to look at the intersection
        // of the ellipse w/ the two verticle edges. 
        /* enclosed by yusu -- to simplify it ... 
          decision_twoedge((*m_P)[i], (*m_P)[i+1], (*m_Q)[0], (*m_Q)[1], cellsides);
        if(cellsides[1].ycoord() < 0.0) // empty
            return 0;
        lownew = cellsides[1].xcoord();
        if( i == 0) {// first time, initialize top and down;
            if(cellsides[3].xcoord() > 0.0) 
                return(0);
            else low = lownew;
            if(cellsides[3].ycoord() < 0.0)
                return(0);
        }
        else {
            if(low > cellsides[1].ycoord())
                return 0;
            low = (low < lownew)?lownew : low;
        }
        */
        
        // for 2D.  
        //twoedge_simp((*m_P)[i], (*m_P)[i+1], twosides, (int)(i==0));
        twoedge_simp_3D((*m_P)[i], (*m_P)[i+1], twosides, (int)(i==0));
        if(twosides[1].ycoord() < 0.0)
            return 0;
        lownew = twosides[1].xcoord();
        if(i == 0) {// first time 
            if(twosides[0].ycoord() < 0.0)
                return 0;
            if(twosides[0].xcoord() > 0.0)
                return 0;
            else low = lownew;
        }
        else {
            if(low > twosides[1].ycoord())
                return 0;
            low = (low < lownew)?lownew : low;
        }
    }
    //if(cellsides[1].ycoord() < 1.0)
    if(twosides[1].ycoord() < 1.0)
        return 0;

    return 1;
}

// this is for 2D solution. It is correct. 
void Frechet::setefgh(PointType& q0, PointType& q1)
{
    DType e = q0.xcoord(); DType es = e*e;
    DType f = q0.ycoord(); DType fs = f*f;
    DType g = q1.xcoord(); DType gs = g*g;
    DType h = q1.ycoord(); DType hs = h*h;

    m_ge = g - e;
    m_hf = h - f;
    m_den = es - 2*e*g + gs + fs - 2*h*f + hs;
    m_A1  = es + fs - h*f - e*g;
    m_B1 = 2*(-h*f*e + g*fs - g*h*f + hs*e);
    m_B2 = 2*(-e*g*f + h*es + gs*f - h*e*g);
    m_B3 = -m_hf * m_hf;
    m_B4 = -m_ge * m_ge;
    m_B5 = 2*m_ge*m_hf;
    m_B6 = m_epsilon*m_epsilon * (m_ge*m_ge + m_hf*m_hf) - hs*es - gs*fs + 2*h*f*e*g;

}

// This is 3D solution. It is correct. 
void Frechet::twoedge_simp(PointType& p0, PointType& p1, array<point>& twosides, int bothflag)
{
    twosides.resize(2);
    point zeropoint(0.0, -0.1);

    DType a = p0.xcoord(); 
    DType b = p0.ycoord(); 
    DType c = p1.xcoord(); 
    DType d = p1.ycoord(); 

    DType r1, r2;
    DType A, B; 

    if(bothflag == 1) {
        A  = m_A1 + a*m_ge +b*m_hf;
        B   = a*m_B1 + b*m_B2 + a*a*m_B3 + b*b*m_B4 + a*b*m_B5 + m_B6;
        if(B < 0.0) { // there is no solution!
            twosides[0] = zeropoint; 
        }
        else {
            B = sqrt(B);
            r1 = (A + B) / m_den;
            r2 = (A - B) / m_den;
            if(r1 > 1.0) r1 = 1.0;
            if(r2 < 0.0) r2 = 0.0;
            point tmpp(r2, r1);
            twosides[0] = tmpp;
        }
    }
    else twosides[0] = zeropoint; 

    A = m_A1 + c*m_ge +d*m_hf;
    B = c*m_B1 + d*m_B2 + c*c*m_B3 + d*d*m_B4 + c*d*m_B5 + m_B6;
    if(B < 0.0) {
        twosides[1] = zeropoint;
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / m_den;
        r2 = (A - B) / m_den;
        if(r1 > 1.0) r1 =1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        twosides[1] = tmpp;
    }
}

// This is for 2D solution, I am not sure whethe it is complete correct or not.
// Should use twoedge_simp. 
void Frechet::decision_twoedge(PointType& p0, PointType& p1,\
                               PointType& q0, PointType& q1,\
                               array<point>& cellsides)
{
    cellsides.resize(4);
    point zeropoint(0.0, -0.1);

    DType a = p0.xcoord(); DType as = a*a;
    DType b = p0.ycoord(); DType bs = b*b;
    DType c = p1.xcoord(); DType cs = c*c;
    DType d = p1.ycoord(); DType ds = d*d;
    DType e = q0.xcoord(); DType es = e*e;
    DType f = q0.ycoord(); DType fs = f*f;
    DType g = q1.xcoord(); DType gs = g*g;
    DType h = q1.ycoord(); DType hs = h*h;
    DType t = m_epsilon * m_epsilon;

    DType r1, r2;

    //first compute for p0, and p1, the points on q_0q_1 such that 
    // the distance is epsilon. 
    DType den = es - 2*e*g + gs + fs - 2*h*f + hs;
    DType A1   = es + fs - h*f - e*g;
    DType A    = A1 + a*(g-e) + b*(h-f);
    DType B1 = 2*(-h*f*e + g*fs - g*h*f + hs*e);
    DType B2 = 2*(-e*g*f + h*es + gs*f - h*e*g);
    DType B3 = 2*h*f - hs - fs;
    DType B4 = 2*e*g - es - gs;
    DType B5 = 2*(e-g)*(f-h);
    DType B6 = t * (-2*e*g - 2*h*f + hs + gs + es + fs) - hs*es - gs*fs + 2*h*f*e*g;
    DType B   = a*B1 + b*B2 + as*B3 + bs*B4 + a*b*B5 + B6;
    if(B < 0.0) { // there is no solution!
        cellsides[3] = zeropoint; 
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / den;
        r2 = (A - B) / den;
        if(r1 > 1.0) r1 = 1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        cellsides[3] = tmpp;
    }

    A = A1 + c*(g-e) +d*(h-f);
    B = c*B1 + d*B2 + cs*B3 + ds*B4 + c*d*B5 + B6;
    if(B < 0.0) {
        cellsides[1] = zeropoint;
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / den;
        r2 = (A - B) / den;
        if(r1 > 1.0) r1 =1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        cellsides[1] = tmpp;
    }

    // now compute for q0 q1, the point on p0p1 such that distance is epsilon
    den = as + bs + cs + ds - 2*a*c - 2*b*d;
    A1 = as + bs - a*c - b*d;
    A  = A1 + e*(c - a) + f* (d-b);
    B1 = 2*(-b*d*a + c*bs - c*b*d + ds*a);
    B2 = 2*(d*as - a*c*b - a*c*d + cs*b);
    B3 = 2*b*d - bs - ds;
    B4 = 2*a*c - as - cs;
    B5 = 2*(a-c) * (b-d);
    B6 = t*(as + bs + cs + ds - 2*b*d - 2*a*c) + 2*a*b*c*d - as*ds - cs*bs;
    B = e*B1 + f*B2 + es*B3 + fs*B4 +e*f*B5 + B6;
    if(B < 0.0) {
        cellsides[0] = zeropoint;
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / den;
        r2 = (A - B) / den;
        if(r1 > 1.0) r1 = 1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        cellsides[0] = tmpp;
    }

    A = A1 + g*(c-a) + h*(d-b);
    B = g*B1 + h*B2 + gs*B3 + hs*B4 +g*h*B5 + B6;
    if(B < 0.0) {
        cellsides[2] = zeropoint;
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / den;
        r2 = (A - B) / den;
        if(r1 > 1.0) r1 = 1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        cellsides[2] = tmpp;
    }
}

void Frechet::setghijkl(PointType& q0, PointType& q1)
{
    DType g = q0.xcoord(); DType gs = g*g;
    DType h = q0.ycoord(); DType hs = h*h;
    DType i = q0.zcoord(); DType is = i*i;
    DType j = q1.xcoord(); DType js = j*j;
    DType k = q1.ycoord(); DType ks = k*k;
    DType l = q1.zcoord(); DType ls = l*l;

    m_jg = j - g;
    m_kh = k-h; 
    m_li = l - i;

    m_den = m_jg*m_jg + m_li * m_li + m_kh*m_kh;
    m_A1 = is + gs + hs - i*l - g*j - h*k;
    m_B1 = 2*(j*(hs + is) + g*(ls + ks) - i*l*g - h*k*g - i*j*l - j*h*k);
    m_B2 = 2*(k*(gs + is) + h*(js+ls) - i*l*h - g*j*h - k*i*l - k*g*j);
    m_B3 = 2*(l*(gs + hs)+i*(ks+js)-g*i*j - h*i*k - l*g*j - l*h*k);
    m_B4 = -m_li*m_li - m_kh*m_kh;
    m_B5 = -m_li*m_li - m_jg*m_jg;
    m_B6 = -m_jg*m_jg - m_kh*m_kh;
    m_B7 = 2*m_kh*m_jg;
    m_B8 = 2*m_kh*m_li;
    m_B9 = 2*m_jg*m_li;
    double tmpkgjh = k*g-j*h; 
    double tmpkilh = k*i-l*h;
    double tmpijlg = i*j-l*g;
    m_B10 = m_epsilon*m_epsilon*m_den - tmpkgjh*tmpkgjh - tmpkilh*tmpkilh - tmpijlg*tmpijlg; 

}

// This is 3D solution. It is correct. 
void Frechet::twoedge_simp_3D(PointType& p0, PointType& p1, array<point>& twosides, int bothflag)
{
    twosides.resize(2);
    point zeropoint(0.0, -0.1);

    DType d = p1.xcoord();
    DType e = p1.ycoord();
    DType f = p1.zcoord();

    DType r1, r2;
    DType A, B; 

    if(bothflag == 1) {
        DType a = p0.xcoord(); 
        DType b = p0.ycoord(); 
        DType c = p0.zcoord(); 
        A  = m_A1 + a*m_jg +b*m_kh +c*m_li;
        B   = a*m_B1 + b*m_B2 + c*m_B3 + a*a*m_B4 + b*b*m_B5 + c*c*m_B6 + a*b*m_B7 + b*c*m_B8 + a*c*m_B9 + m_B10;
        if(B < 0.0) { // there is no solution!
            twosides[0] = zeropoint; 
        }
        else {
            B = sqrt(B);
            r1 = (A + B) / m_den;
            r2 = (A - B) / m_den;
            if(r1 > 1.0) r1 = 1.0;
            if(r2 < 0.0) r2 = 0.0;
            point tmpp(r2, r1);
            twosides[0] = tmpp;
        }
    }
    else twosides[0] = zeropoint; 

    A = m_A1 + d*m_jg +e*m_kh + f*m_li;
    B = d*m_B1 + e*m_B2 + f*m_B3 + d*d*m_B4 + e*e*m_B5 + f*f*m_B6 + d*e*m_B7 +e*f*m_B8 + d*f*m_B9 + m_B10;
    if(B < 0.0) {
        twosides[1] = zeropoint;
    }
    else {
        B = sqrt(B);
        r1 = (A + B) / m_den;
        r2 = (A - B) / m_den;
        if(r1 > 1.0) r1 =1.0;
        if(r2 < 0.0) r2 = 0.0;
        point tmpp(r2, r1);
        twosides[1] = tmpp;
    }
}

