/******* Yusu      -- March 23 **************************/
/******* Frechet.h -- Compute the frechet distance ******/
/*******              between two polygonal curve  ******/

#ifndef _FRECHET_H_
#define _FRECHET_H_

#include <stdio.h>
#include <stdlib.h>

// LEDA include


// other include
#include "datatype.h"

class Frechet{
public:
    Frechet();
    ~Frechet();

    void RemoveAll();
    
    void setPolyline(Polyline& P, Polyline& Q) {
        m_P = &P; m_Q = &Q; 
    }
    void setPolyline(int index, Polyline& P) {
        if (index == 0) 
            m_P = &P; 
        else m_Q = &P; 
    } 

    // decision version of frechet. 
    // return 1 if frechet(P, Q) <= epsilon
    // otherwise return 0. 
    int Decision(Polyline& P, Polyline& Q, DType epsilon);
    int Decision(DType epsilon);

private:
    //data members
    Polyline *m_P;
    Polyline *m_Q;

    DType m_epsilon;

    DType m_A1, m_B1, m_B2, m_B3, m_B4, m_B5, m_B6, m_ge, m_hf, m_den;
    DType m_B7, m_B8, m_B9, m_B10, m_jg, m_kh, m_li;

    //function members
    void decision_twoedge(PointType& p0, PointType& p1,\
                               PointType& q0, PointType& q1,\
                                  array<point>& cellsides); 
    void setefgh(PointType& q0, PointType& q1); 
    void setghijkl(PointType& q0, PointType& q1);
    void twoedge_simp(PointType& p0, PointType& p1, array<point>& twosides, int bothflag);
    void twoedge_simp_3D(PointType& p0, PointType& p1, array<point>& twosides, int bothflag);

};



#endif /*_FRECHET_H_ */
