#include <fstream.h>
#include <time.h>

timeval *timevalue ;

void output_simp_curve( array<d3_point>* pPoly ) ; 

void assignColor(int index)
{ 
  glColor3fv(ColorMap[index]);
} 

void print_point(DType* pts)
{
  cout << "print_point: " << pts[0] << " " << pts[1] << " " << pts[2] << endl;
}

void print_vector(DType v[], int n)
{
  for(int i = 0; i < n; i ++) 
   cout << " *" << i << ": " << v[i];
  cout << endl;
}

void reset()
{
  X_Trans = 0.0;
  Y_Trans = 0.0;
  Z_Trans = 0.0;
  XAngle = YAngle = ZAngle = 0.0;
}

void setVariables()
{
    // flags

    // variables
  X_Min = 0.0;
  Z_Min = 0.0;
  X_Max = 0.0;
  Z_Max = 0.0;
  Y_Min = 0.0;
  Y_Max = 0.0;
  X_Mid = 0.0;
  Y_Mid = 0.0;
  Z_Mid = 0.0;
  XDelta = 0.0;
  YDelta = 0.0;
  ZDelta = 0.0;

  poly_flag = 1;
  simp_flag = 0;
  p_g_simp_poly = NULL;

  reset();
}

void print_help()
{
  cout << "\t\t **** Functions of Keys **** " << endl;
  cout << "\t 'q'	quit " << endl;
  cout << "\t 'r'	read the input polyline file " << endl;
  cout << "\t 's'       simplify the polyline " << endl;
  cout << "\t 'e'       exactly simplify the polyline " << endl;
  cout << "\t 'd'       douglas-peucker simplify the polyline " << endl;
  cout << "\t 't'       enable/disable the drawing of original curve " << endl;
  cout << "\t 'f'       enable/disable the drawing of simplified curve" << endl;
  cout << "\t 'i'       display the statistics of current simplification " << endl;
  cout << "\t 'h' 	print the help list " << endl;
  cout << "\t\t      *****************      " << endl;
  cout << "\t\t **** Functions of Mouse buttons **** " << endl;
  cout << "\t\t      *****************      " << endl;
}

void cal_Cross(vector& v1, vector& v2, vector& v3)
{
    v1[0] = v2[1]*v3[2] - v2[2]*v3[1];
    v1[1] = v2[2]*v3[0] - v2[0]*v3[2];
    v1[2] = v2[0]*v3[1] - v2[1]*v3[0];
}

double cal_Dot(vector& v1, vector& v2)
{
    return( v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2]);
}

/************ read input curve to simplify ****************/
void read_opencurve( )
{
    cout << " Input the filename : " << endl;
    char filename[80]; 
    cin >> filename;

    FILE* fp = fopen(filename, "r");
    if(fp == NULL) {
        cout << " can't open file " << filename << endl;
        return;
    }
    
    char aline[200], stmp[200];
    int dtmp;
    float x, y, z;
    fgets(aline, 200, fp);
    sscanf(aline, "%s%d", stmp, &g_polysize);
    g_input_poly.resize(g_polysize);
    int count = 0;
    while(fgets(aline, 200, fp) != NULL) {
        if(sscanf(aline, "%d %f %f %f", &dtmp, &x, &y, &z) == 4) {
        //if(sscanf(aline, "%d %f %f %f", &dtmp, &x, &z, &y) == 4) {
        //if(sscanf(aline, "%d %f %f %f", &dtmp, &y, &z, &x) == 4) {
            if(count == 0) {
                X_Min = X_Max = x;
                Y_Min = Y_Max = y;
                Z_Min = Z_Max = z;
            }
            else {
                if(x < X_Min) X_Min = x;
                else if(x > X_Max) X_Max = x;
                if(y < Y_Min) Y_Min = y;
                else if(y > Y_Max) Y_Max = y;
                if(z < Z_Min) Z_Min = z;
                else if(z > Z_Max) Z_Max = z;
            }
            d3_point p((double)x,(double)y,(double)z);
            g_input_poly.set(count, p);
            count ++;
        }
    }
    X_Mid = (X_Min + X_Max) / 2.0;
    Y_Mid = (Y_Min + Y_Max) / 2.0;
    Z_Mid = (Z_Min + Z_Max) / 2.0;
    XDelta = (X_Max - X_Min) / 20.0;
    YDelta = (Y_Max - Y_Min) / 20.0;
    ZDelta = (Z_Max - Z_Min) / 20.0;

    fclose(fp);
    cout << "Read file succeeded... " << endl;
    
    // init the simplification tool
    simp_tool.input_curve(g_input_poly);
}

void simp_curve_uniform()
{
    cout << "Input the error " << endl;
    cout << "> ";
    DType epsilon;
    cin >> epsilon;

    p_g_simp_poly = &(simp_tool.simp_curve_uniform(epsilon)); 
}

void simp_curve_hausdorff()
{
    cout << "Input the error " << endl;
    cout << "> ";
    DType epsilon;
    cin >> epsilon;

    p_g_simp_poly = &(simp_tool.simp_curve_hausdorff(epsilon)); 
}

void simp_curve_approx()
{
    cout << "Input the error " << endl;
    cout << "> ";
    DType epsilon;
    cin >> epsilon;

    p_g_simp_poly = &(simp_tool.simp_curve_approx(epsilon)); 
    output_simp_curve( p_g_simp_poly ); 
}

void simp_curve_exact()
{
    cout << "Input the error " << endl;
    cout << "> ";
    DType epsilon;
    cin >> epsilon;

    p_g_simp_poly = &(simp_tool.simp_curve_exact(epsilon)); 

}

void simp_curve_dp()
{
    cout << "Input the error " << endl;
    cout << "> ";
    DType epsilon;
    cin >> epsilon;

    p_g_simp_poly = &(simp_tool.simp_curve_dp(epsilon)); 

}

// print the statistics of the current simplification. 
void print_stat()
{
    cout << " *********** Statistics ************ " << endl;
    cout << " *** #vertices of original curve: " << simp_tool.curve_len() << endl;
    cout << " *** #vertices of simp. curve;    " << simp_tool.curr_simp() << endl;
    cout << " *** Error : " << simp_tool.get_error() << endl;
    cout << " *********** End        ************ " << endl;

}

void output_simp_curve(void ) {

}

// display
void display_poly(array<d3_point>* pPoly, int colorid)
{
    if ( pPoly == NULL ) {
      cout << "No simplified curve yet.\n" ; 
      return;  
    }

    // first draw the points
    int i;
    assignColor(Red); 
    glPointSize(3);
    glBegin(GL_POINTS);
    for(i = 0; i < pPoly->size(); i ++) {
        glVertex3f((*pPoly)[i].xcoord(), (*pPoly)[i].ycoord(), (*pPoly)[i].zcoord());
    }
    glEnd(); 

    // then draw the polyline.
    glLineWidth(2);
    assignColor(colorid); 
    glBegin(GL_LINES);
    for(i = 0; i < pPoly->size()-1; i ++) {
        glVertex3f((*pPoly)[i].xcoord(), (*pPoly)[i].ycoord(), (*pPoly)[i].zcoord());
        glVertex3f((*pPoly)[i+1].xcoord(), (*pPoly)[i+1].ycoord(), (*pPoly)[i+1].zcoord());
    }
    glEnd(); 

}

void read_opencurve(char *filename)
{
    FILE* fp = fopen(filename, "r");
    if(fp == NULL) {
        cout << " can't open file " << filename << endl;
        return;
    }
    
    char aline[200], stmp[200];
    int dtmp;
    float x, y, z;
    fgets(aline, 200, fp);
    sscanf(aline, "%s%d", stmp, &g_polysize);
    g_input_poly.resize(g_polysize);
    int count = 0;
    while(fgets(aline, 200, fp) != NULL) {
        if(sscanf(aline, "%d %f %f %f", &dtmp, &x, &y, &z) == 4) {
        //if(sscanf(aline, "%d %f %f %f", &dtmp, &x, &z, &y) == 4) {
        //if(sscanf(aline, "%d %f %f %f", &dtmp, &y, &z, &x) == 4) {
            if(count == 0) {
                X_Min = X_Max = x;
                Y_Min = Y_Max = y;
                Z_Min = Z_Max = z;
            }
            else {
                if(x < X_Min) X_Min = x;
                else if(x > X_Max) X_Max = x;
                if(y < Y_Min) Y_Min = y;
                else if(y > Y_Max) Y_Max = y;
                if(z < Z_Min) Z_Min = z;
                else if(z > Z_Max) Z_Max = z;
            }
            d3_point p((double)x,(double)y,(double)z);
            g_input_poly.set(count, p);
            count ++;
        }
    }
    X_Mid = (X_Min + X_Max) / 2.0;
    Y_Mid = (Y_Min + Y_Max) / 2.0;
    Z_Mid = (Z_Min + Z_Max) / 2.0;
    XDelta = (X_Max - X_Min) / 20.0;
    YDelta = (Y_Max - Y_Min) / 20.0;
    ZDelta = (Z_Max - Z_Min) / 20.0;

    fclose(fp);
    cout << "Read file succeeded... " << endl;
    
    // init the simplification tool
    simp_tool.input_curve(g_input_poly);
}

void starttimer() 
{
  gettimeofday(timevalue, NULL) ;
} 

float endtimer() 
{
  timeval *t = new timeval() ;
  gettimeofday(t, NULL) ;
  return ( (double)( (t->tv_sec - timevalue->tv_sec)*1000000 + t->tv_usec - timevalue->tv_usec ) )/ (1000000) ; 
}

void batch_mode_file()
{
    int i,j, k ;
    timevalue = new timeval( ) ;

    DType E [] = { 0.01, 0.03, 0.05, 0.08, 0.1, 0.3, 0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 5.0 } ;

    DType epsilon;

    read_opencurve("../../data/1aus_50000.mol") ; 
    for ( j= 0; j < 13; j++ ) {
      cout << E[j] ; 
      starttimer( ) ; 
      for (k=0;k<10;k++)  p_g_simp_poly = &(simp_tool.simp_curve_approx(E[j])); 
      cout << " " << (float)(endtimer( ))/10 << endl ; 

      //starttimer( ) ; 
      //p_g_simp_poly = &(simp_tool.simp_curve_dp(E[j])); 
      //cout << " " << endtimer( ) << endl ; 
    }
}


void batch_mode_time()
{
    int i,j ;
    ofstream fout("output.time");   
    clock_t cl;

    char **files = new char *[5]; 
    for (i=0;i<5;i++) files[i] = new char[20]; 
    strcpy( files[0], "../data/1aus_50000.mol"); 
    strcpy( files[1], "../data/1cdk_10000.mol"); 
    strcpy( files[2], "../data/1cdk_3000.mol"); 
    //strcpy( files[3], "../data/f8_1000_1.mol"); 

    DType E [] = { 0.1, 0.5, 1.0, 2, 5.0, 20.0 }; 

    DType epsilon;

    for (i=0;i<5;i++) fout << files[i] << " & " ; 
    fout << "\\\\ \\hline" << endl;  

    for ( j= 0; j < 6; j++ ) {
      fout << E[j] ; 
      for ( i=0;i<3;i++ ) {
        read_opencurve(files[i]);
        cl = clock(); 
        p_g_simp_poly = &(simp_tool.simp_curve_approx(E[j])); 
        fout << " & " << (clock()-cl)/((double)CLOCKS_PER_SEC) ;

        cl = clock(); 
        p_g_simp_poly = &(simp_tool.simp_curve_dp(E[j])); 
        fout << " & " << (clock()-cl)/((double)CLOCKS_PER_SEC) ;
      }
      fout << " \\\\ " << endl;  
    }
}

void batch_mode_size()
{
    int i,j ;
    ofstream fout("output.size");   
    clock_t cl;

    char **files = new char *[5]; 
    for (i=0;i<5;i++) files[i] = new char[100]; 

    strcpy( files[0], "../../../../Data/stock/d7103.transporttele.out"); 
    strcpy( files[1], "../../../../Data/stock/d9303.telecommunicationbiotech.out"); 

    DType E [] = { 0.05, 0.5, 1.2, 3.0, 5.0 }; 

    DType epsilon;

    for (i=0;i<2;i++) {
      fout << files[i] << " & " ; 
      fout << "\\\\ \\hline" << endl;  

      for ( j = 0; j < 5; j++ ) {
        fout << E[j] ; 
        for ( i=0;i<1;i++ ) {
          read_opencurve(files[i]);
          p_g_simp_poly = &(simp_tool.simp_curve_approx(E[j])); 
          fout << " & " << p_g_simp_poly->size() ; 
          p_g_simp_poly = &(simp_tool.simp_curve_exact(E[j])); 
          fout << " & " << p_g_simp_poly->size() ; 
        }
        fout << " \\\\ " << endl;  
      }
   }  
}

void output_simp_curve( array<d3_point>* pPoly ) {
  int i, j, k;

  ofstream fout("simpcurve.txt"); 
  for(i = 0; i < pPoly->size(); i++) {
    fout << (*pPoly)[i].xcoord() << " " <<  (*pPoly)[i].ycoord() << " " <<  (*pPoly)[i].zcoord() << endl ;
  }
  fout.close(); 
}
