#include "main.h"

void init();
void setLight();
void clean_up();
void Display();

void init()
{
  // initialization for OpenGL:
  glBlendFunc(GL_SRC_ALPHA_SATURATE, GL_ONE);
  glDepthFunc(GL_LEQUAL);
  glCullFace(GL_BACK);
  glClearColor(0, 0, 0, 0);   /* Background color = Black*/
  glShadeModel(GL_SMOOTH);    /* Smooth shading */
  setLight();

  setVariables();
}

void setLight(void)
{
   glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
   glLightfv(GL_LIGHT0, GL_POSITION, light_position);
}

void clean_up()
{


}

void KeyPress(unsigned char key, int x, int y)
{
  int flag = 1;

  switch (key)
  {
  case 'q': clean_up();
      exit(0);
      break;
  case 'h': print_help();
      break;
  case 'n': Z_Trans += 15;
      break;
  case 'm': Z_Trans -= 15;
      break;
  case 'r': // read the input curve
      read_opencurve();
      break;

  case 'u': // uniform simplify the curve. 
      printf("Simplifying using Uniform\n") ;   
      simp_curve_uniform(); 
      print_stat();
      break; 
  case 's': // approximately simplify the curve. 
      printf("Simplifying using Frechet approximate\n") ;   
      simp_curve_approx(); 
      print_stat();
      break; 
  case 'e': // exactly simplify the curve. 
      printf("Simplifying using Frechet exact\n") ;   
      simp_curve_exact(); 
      break; 
  case 'd': // douglas peucker simplify the curve. 
      printf("Simplifying using Douglas Peucker\n") ;   
      simp_curve_dp(); 
      break; 
  case 'c': // hausdorff simplify the curve. 
      printf("Simplifying using Hausdorff approximate\n") ;   
      simp_curve_hausdorff(); 
      break; 
  case 'k': // batch mode.
      printf("Outputting simplified curve\n") ; 
      output_simp_curve( p_g_simp_poly ); 
      break; 
  case '1': // batch mode.
      batch_mode_time(); 
      break; 
  case '2': // batch mode.
      batch_mode_size(); 
      break; 
  case '3': // particular file.
      batch_mode_file(); 
      break; 
  case 't':
      poly_flag = 1 - poly_flag;
      break;
  case 'f':
      simp_flag = 1 - simp_flag; 
      break;
  case 'i': // display statistics
      print_stat();
      break;
  case 27: exit(0);  
  default:  break;
  }
  
  glutPostRedisplay();
}

void MouseMotion(int x, int y)
{
  GLdouble mvmatrix[16],projmatrix[16];
  GLint viewport[4];
  GLint realy;
  GLdouble w[3];

  glGetIntegerv(GL_VIEWPORT,viewport);
  realy = viewport[3] - (GLint) y -1;
//  cout << "x, y: " << x << " " << realy << endl;
  switch(MoveButton) {
    case 0: // left button
	  // rotating..
	XAngle += (Old_y - realy) * XRT_Scale;
	YAngle += (x - Old_x) * YRT_Scale;
//        cout << "XAngle: "<< XAngle << " YAngle: "<< YAngle << endl;
      	break;
    case 1: // middle button 
	  // shifting..
 	X_Trans += (x - Old_x) * XSF_Scale;	
  	Y_Trans += (realy - Old_y) * YSF_Scale;
//        cout << "X_Trans: "<< X_Trans << " Y_Trans: "<< Y_Trans << endl;
	break;
    case 2: // right button
	  // nothing currently
	break;
    case -1: 
	  // doing nothing
	break;
    default: cout << "Can't be this button. " << endl;
        break;
   }
     // update the old_coordinates.
   Old_x = x;
   Old_y = realy;
  
   glutPostRedisplay();
}


void MousePress(int button, int state, int x, int y) 
{
  GLdouble mvmatrix[16],projmatrix[16];
  GLint viewport[4];
  GLint realy;

  glGetIntegerv(GL_VIEWPORT,viewport);
  MoveButton = -1; 
  switch(button) {
  case GLUT_LEFT_BUTTON:
   MoveButton = 0;
  case GLUT_MIDDLE_BUTTON:
   if(MoveButton == -1)
	MoveButton = 1;
   if (state == GLUT_DOWN)  {
    Old_y = viewport[3] - (GLint) y -1;
    Old_x = x;
//	cout << "start: (x, y) " << x << ", " << Old_y << endl;
   }
   if (state == GLUT_UP) {

   }
   break;
  case GLUT_RIGHT_BUTTON:
   MoveButton = 2;
   break;
  default: break;
  }

  glutPostRedisplay();
}  


void Display()  
{    
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
  glColor3f(0.0, 0.0, 0.0); 
  glEnable(GL_DEPTH_TEST);
  
  glMatrixMode(GL_MODELVIEW);
 
  glLoadIdentity();
  glTranslatef(X_Trans, Y_Trans, Z_Trans);
  glTranslatef(0, 0, -1.5*(Z_Max - Z_Min));
  glRotatef(XAngle, 1.0, 0.0, 0.0);
  glRotatef(YAngle, 0.0, 1.0, 0.0);
  glRotatef(ZAngle, 0.0, 0.0, 1.0);
  glTranslatef(-X_Mid, -Y_Mid, Z_Mid);

  if(poly_flag) 
      display_poly(&(g_input_poly), Yellow);
  if(simp_flag) 
      display_poly(p_g_simp_poly, White); 

  glDisable(GL_DEPTH_TEST);

  glutSwapBuffers();
  glFlush();

}

void Reshape(int w, int h)
{ 
  glutSetWindow(Mainwin);
  
  glViewport(0,0,(GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(90.0, (GLfloat)w/(GLfloat)h, ZNEAR, ZFAR);
     
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}    

static void Idle()
{ 
    if(MoveFlag) {
//        Display();
    }
}

void MainMenu(int val)
{

  switch(val)
  {
  case QUIT_CONST:  clean_up();
      exit(0);
      break;
  case RESET_CONST: reset();
      break;
  case HELP_CONST:  print_help();
      break;
  }

  glutPostRedisplay();
}

void SetupMenu()
{ 
  glutCreateMenu(MainMenu);
  glutAddMenuEntry("Reset",RESET_CONST);
  glutAddMenuEntry("Help",HELP_CONST);
  glutAddMenuEntry("Quit",QUIT_CONST);
  
  glutAttachMenu(GLUT_RIGHT_BUTTON);
}

int main(int argc, char **argv)
{
  glutInit(&argc, argv);
  glutInitWindowPosition(100,100);

  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB | GLUT_ALPHA);
  glutInitWindowSize(512, 512);
  glutCreateWindow(" display curve ");
  Mainwin = glutGetWindow();
  init();
  glutDisplayFunc(Display); 
  glutReshapeFunc(Reshape);
  //glutIdleFunc(Idle);
  SetupMenu();
  glutKeyboardFunc(KeyPress);
  glutMouseFunc(MousePress);
  glutMotionFunc(MouseMotion);
  
  if ( argc > 1 ) read_opencurve(argv[1]);
  glutMainLoop();

  return 0;
}




