#ifndef _MC_MAIN_H
#define _MC_MAIN_H

// system includes:
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <fcntl.h>
#include <time.h>  
#include <GL/glut.h>

// STL include
// #include <stl.h>
// #include <stl_queue.h>

// LEDA include
#include <LEDA/list.h>
#include <LEDA/geo_alg.h>
#include <LEDA/array.h>
#include <LEDA/d3_point.h>
#include <LEDA/d3_segment.h>
#include <LEDA/d3_line.h>
#include <LEDA/d3_plane.h>
#include <LEDA/map.h>
#include <LEDA/vector.h>

// Others
#include "datatype.h"
#include "simp.h"

// define const.
#define QUIT_CONST    100
#define RESET_CONST   102
#define HELP_CONST    108
#define ZNEAR         1.0
#define ZFAR          100000.0		

#define MY_PI         3.1415926
#define MY_2PI        6.2831852
#define MY_EPSILON    0.000001

typedef float VECTOR3D[3];

// external varialbles:

// constants:

// enum:
enum ColorType {
  White = 0, Red = 1, Green = 2, Blue = 3, Yellow = 4, VFColor = 5, VDColor = 6, BuildColor = 7, Object = 8};

// static variables:
static float ColorMap[12][3] = {
{1.0, 1.0, 1.0},
{1.0, 0.0, 0.0},
{0.0, 1.0, 0.0},
{0.0, 0.0, 1.0},
{1.0, 1.0, 0.0},
{1.0, 0.0, 1.0},
{0.0, 1.0, 1.0},
{0.5, 0.5, 1.0},
{1.0, 0.5, 0.5},
{1.0, 0.5, 0.0},
{0.8, 0.3, 0.6},
{1.0, 1.0, 1.0}
};

// variables:

  // flags:
int DrawFlag[10] = {0,0,0,0,0,0,0,0,0,0};

  // Window handler:
int 	Mainwin = 0;

  // related to lights:
GLfloat light_ambient[] = { 0.3, 0.3, 0.3, 1.0 };
GLfloat light_diffuse[] = { 1.0, 0.0, 1.0, 1.0 };
GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat light_position[] = {-0.3, 0.5, 1.0, 0.0 };
GLfloat color_mat[4];

  // related to the data:
DType      X_Min, X_Max, X_Mid;
DType      Y_Min, Y_Max, Y_Mid;
DType      Z_Min, Z_Max, Z_Mid;
DType      XDelta, YDelta, ZDelta;

  // related to display:
float	X_Trans, Y_Trans, Z_Trans, XAngle;
float   YAngle, ZAngle;
float   S_XAngle, S_YAngle, S_ZAngle;
float	S_X_Trans, S_Y_Trans, S_Z_Trans;
VECTOR3D StartPt, EndPt, CenterPt;
DType   RotMat[16];
DType   IdenMat[16] = { 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0};

  // related to view:
VECTOR3D ViewOrigin = {0, 0, 0};
VECTOR3D ViewPt;
int      Old_x, Old_y;
int      MoveButton = -1;
float    XRT_Scale = 0.14;
float    YRT_Scale = 0.08;
float    XSF_Scale = 0.56;
float    YSF_Scale = 0.60;

// related to dna movements;
int    Cur_Time = 0;
int    RodFlag = 0;
int    PairFlag = 1;
int    MoveFlag = 0;
int    KineticDna = 0;


// related to LEDA
array<d3_point> g_input_poly;
array<d3_point>* p_g_simp_poly = NULL;

int g_polysize = 0;

// for display:
int poly_flag = 1;
int simp_flag = 0;
// related to simplification
Simp_tool simp_tool;

#include "function.h"

#endif




