/****** Modified by yusu on Nov. 8th                **************/
/****** Mainly add validity check into the edge contraction ******/
/****** I would try to mark it where I changed      **************/

#include "graph.h"
#include <LEDA/graph.h>
#include <LEDA/templates/shortest_path.t>
#include <LEDA/d3_segment.h>
#include <LEDA/d3_line.h>
#include <math.h>
#include "algo.hpp" 
#include <stdio.h> 

#include "simp.h"

double distance_point_to_seg( d3_segment & s, d3_point & p)
{
  double dp1p, dp2p, dp1p2, dpl;
  d3_line l(s) ;

  d3_point p1 = s.source() ;
  d3_point p2 = s.target() ;

  dp1p = p1.sqr_dist(p) ;
  dp2p = p2.sqr_dist(p) ;
  dpl  = l.sqr_dist(p);
  dp1p2  = p1.sqr_dist(p2);

  if ( (dp1p2 > dp1p - dpl) && (dp1p2 > dp2p - dpl) )
  {
    // point p inside the line segment s
    return sqrt( dpl ) ;
  }

  else {
    // point p outside the line segment s
    return ( dp1p < dp2p ? sqrt(dp1p) : sqrt(dp2p) ) ;
  }
}

Simp_tool::Simp_tool()
{
    init();
}

Simp_tool::Simp_tool(array<d3_point>& i_curve)
{
    init();
    input_curve(i_curve);
}

Simp_tool::~Simp_tool()
{
    reset();
}

void Simp_tool::init()
{
    m_error = 0.0;
    m_vertex = 0;
    m_size = 0;
}

void Simp_tool::reset()
{
    m_original.resize(1);
    m_cur_curve.resize(1);
    init();
}

/*** Convert the input array in m_src_curve and m_cur_curve. ****/
void Simp_tool::input_curve(array<d3_point>& i_curve)
{
    reset();
    m_original =  i_curve;
    m_cur_curve = i_curve; // m_cur_curve is initialized to the original curve.
    m_size = m_original.size(); 
    m_vertex = m_size;
    m_error = 0.0;
}

array<PointType>& Simp_tool::getPi(int low, int high, array<PointType>& P)
{
    P.resize(high - low + 1);
    for(int i = low; i <= high; i ++)
        P[i-low] = m_original[i];

    return P; 
}

array<PointType>& Simp_tool::getSeg(int low, int high, array<PointType>& Q)
{
    Q.resize(2);
    Q[0] = m_original[low];
    Q[1] = m_original[high];
    return Q; 
}

array<d3_point>& Simp_tool::simp_curve_approx(DType epsilon)
{
    m_error = epsilon;
    m_cur_curve.resize(1);
    
    int count = 0;
    int low, high, mid;
    int decision; 
    //list<int> tmplist;
    array<int> tmparray(m_size);
    Frechet frechet;

    array<PointType> P, Q; 
    //tmplist.push(count); 
    m_vertex = 0; count = 0;
    tmparray[m_vertex] = count; m_vertex ++;
    while (count < m_size-2) {
        low = count; 
        high = count + 2;
        getPi(count, high, P);
        getSeg(count, high, Q); 
        if(count == 75)
            count = 75;
        decision = frechet.Decision(P, Q, m_error); 
        while( (high < (m_size-1)) && (decision == 1)) {
            low = high; 
            high = 2*high - count;
            if(high >= m_size)
                high = m_size - 1;
            getPi(count, high, P);
            getSeg(count, high, Q); 
            decision = frechet.Decision(P, Q, m_error);
        }
        if(decision == 1)
            low = high;
        else {
            while(low < (high - 1)) {
                mid = (low + high) / 2;
                if(mid == (count +1))
                    low = mid;
                else {
                    getPi(count, mid, P);
                    getSeg(count, mid, Q); 
                    if(frechet.Decision(P, Q, m_error) == 1)
                        low = mid;
                    else high = mid;
                }
            }
        }
        count = low;
        //tmplist.push(count); 
        tmparray[m_vertex] = count; m_vertex ++;
    }

    if(count != (m_size-1)) {
        //tmplist.push((m_size-1));
        tmparray[m_vertex] = m_size-1; m_vertex ++;
    }

    // convert tmplist to m_cur_curve. 
    //m_vertex = tmplist.size();
    m_cur_curve.resize(m_vertex);
    count = 0;
    /*
    while(!tmplist.empty()) {
        m_cur_curve[count] = m_original[tmplist.pop_back()];
        count ++;
    }
    */
   
    for(count = 0; count < m_vertex; count ++)
        m_cur_curve[count] = m_original[tmparray[count]];

//    cout << "The size of the simplified curve is " << count <<" ( " << m_size << " ) " << endl; 
    //printf("simplified curve: %d\n", m_vertex) ; 

    return m_cur_curve;
}

array<d3_point>& Simp_tool::simp_curve_exact(DType epsilon)
{
    int i; 
    int decision; 
    array<int> tmparray(m_size);
    Frechet frechet;
    array<PointType> P, Q; 

    m_error = epsilon;
    m_cur_curve.resize(1);
    
    m_vertex = 0;

    Graph *egraph = new Graph(m_size+1); 

    for (int i=0;i<m_size;i++) egraph->addNode(i) ;  

    for (int i=0;i<m_size-1;i++) {
      egraph->addEdge(i+1, i) ; 
      //cout << "Adding edge: " << i << ", " << i+1 << endl;  
      for (int j=i+2;j<m_size;j++) {
        getPi(i, j, P);
        getSeg(i, j, Q); 
        decision = frechet.Decision(P, Q, m_error); 
   	if ( decision ) {
   	  egraph->addEdge(j, i) ; 
  	  //cout << "Adding edge: " << i << ", " << j << endl;  
	}
      }
    }

    cout << "Number of graph edges: " << egraph->size() << endl;  

    leda_edge e ;
    leda_node start ; 

    leda_edge_array<int> weights( egraph->seggraph,  1 ) ; 

    forall_edges( e, egraph->seggraph ) weights[e] = 1 ; 
    leda_node_array<leda_edge> pred( egraph->seggraph ) ;  
    leda_node_array<int> dist( egraph->seggraph ) ;  

    start = egraph->getnode(m_size-1); 
    DIJKSTRA_T( egraph->seggraph, start, weights, dist, pred) ; 

    start = egraph->getnode(0); 
    i = dist[start]; 
    if ( dist[ start ] != 0 ) {
    cout << "Succesfully computed shortest path of size: " << i << endl ; 
      tmparray[m_vertex++] = index(start); 
      for (; index(start) != m_size-1; ) {
        start = source(pred[start]) ;  
        //cout << index(source(pred[start])) << ", " << index(target(pred[start])) <<  endl; 
        tmparray[m_vertex++] = index(start); 
      }
    }
    else {
      cout << "Error: Shortest path of size 0!\n" ; 
      exit(0); 
    }

    m_cur_curve.resize(m_vertex);
    i = 0;
   
    for(i = 0; i < m_vertex; i++)
        m_cur_curve[i] = m_original[tmparray[i]];

    cout << "The size of the simplified curve is " << i <<" ( " << m_size << " ) " << endl; 

    return m_cur_curve;
}

array<d3_point>& Simp_tool::simp_curve_dp(DType epsilon)
{
    int i; 
    int decision; 
    int * tmparray;

    m_error = epsilon;
    m_cur_curve.resize(1);

    m_vertex = simp_curve_dp(m_error, 0, m_size-1, tmparray); 
    //m_vertex = tmparray.size() ; 

    m_cur_curve.resize(m_vertex);
   
    for(i = 0; i < m_vertex; i++)
        m_cur_curve[i] = m_original[tmparray[i]];

//    cout << "The size of the simplified curve is " << i <<" ( " << m_size << " ) " << endl; 
    //printf("simplified curve: %d\n", m_vertex) ; 

    return m_cur_curve;
}

long Simp_tool::simp_curve_dp(DType epsilon, int low, int high, int * & arr)
{
  if ( high == low + 1 ) {
    arr =  new int[2] ; 
    arr[0] = low ; 
    arr[1] = high ; 
    return 2 ; 
  }

  int i, maxindex, count ;
  d3_segment s( m_original[low], m_original[high] ) ; 

  maxindex = low; 
  for (i=low+1;i<high;i++) {
    if ( distance_point_to_seg( s, m_original[i] ) > distance_point_to_seg( s, m_original[maxindex] ) )  
      maxindex = i ;
  }

  if ( distance_point_to_seg( s, m_original[maxindex] )  <= epsilon ) {
    arr =  new int[2] ; 
    arr[0] = low ; 
    arr[1] = high ; 
    return 2 ; 
  }

  int * arr1;
  int * arr2;
  long arr1size, arr2size; 

  arr1size = simp_curve_dp( epsilon, low, maxindex, arr1 ) ; 
  arr2size = simp_curve_dp( epsilon, maxindex, high, arr2 ) ; 

  arr = new int[arr1size-1 + arr2size] ; 

  count = arr1size-1; 
  for (i=0;i<count;i++) 
    arr[i] = arr1[i] ;   

  maxindex = count; 
  count = arr2size ; 
  for (i=0;i<count;i++) 
    arr[i+maxindex] = arr2[i] ;   
  delete[] arr1 ; 
  delete[] arr2 ; 
  return arr1size-1 + arr2size ; 
}

array<d3_point>& Simp_tool::simp_curve_hausdorff(DType epsilon)
{
    int i; 
    array<int> tmparray(m_size+1);

    m_error = epsilon;
    m_cur_curve.resize(1);

    pointset *input = new pointset(m_size+1) ; 

    for (i=0;i<m_size;i++) {
       input->push( m_original[i].xcoord(), m_original[i].ycoord() ) ;  
    }

    m_vertex = simplifyApproximation( input, tmparray, m_error ) ; 

    m_cur_curve.resize(m_vertex);
   
    for(i = 0; i < m_vertex; i++)
    {
        m_cur_curve[i] = m_original[tmparray[i]];  
    }

    //printf("simplified curve: %d\n", m_vertex) ; 
    return m_cur_curve;
}

array<d3_point>& Simp_tool::simp_curve_uniform(DType epsilon)
{
    int i; 
    array<int> tmparray(m_size+1);

    m_error = epsilon;
    m_cur_curve.resize(1);

    pointset *input = new pointset(m_size+1) ; 

    for (i=0;i<m_size;i++) {
       input->push( m_original[i].xcoord(), m_original[i].ycoord() ) ;  
    }

    m_vertex = simplifyUniform( input, tmparray, m_error ) ; 

    m_cur_curve.resize(m_vertex);
   
    for(i = 0; i < m_vertex; i++)
    {
        m_cur_curve[i] = m_original[tmparray[i]];  
    }

    return m_cur_curve;
}
