/***** by yusu --- Apr. 1st, 2002  *********************************
  This class provides a curve-simplification tool, 
  using Frechet error metrics... 
****************************************************************/
#ifndef _SIMP_H_
#define _SIMP_H_

#include <stdio.h>
#include <stdlib.h>

//LEDA include
#include <LEDA/list.h>
#include <LEDA/array.h>
#include <LEDA/d3_point.h>
#include <LEDA/d3_plane.h>
#include <LEDA/p_queue.h>

// other
#include "datatype.h"
#include "frechet.h"

class Simp_tool{

public:
    Simp_tool();
    Simp_tool(array<d3_point>& i_curve);
    ~Simp_tool();

    //input the source polygonal chain
    void input_curve(array<d3_point>& i_curve);

    //output current approximate simplified chain under uniform cone method with a required vertex number
    array<d3_point>& simp_curve_uniform(DType epsilon);

    //output current approximate simplified chain with a required vertex number
    array<d3_point>& simp_curve_approx(DType epsilon);

    //output current exact simplified chain with a required vertex number
    array<d3_point>& simp_curve_exact(DType epsilon);

    //output current douglas peucker simplified chain with a required vertex number
    array<d3_point>& simp_curve_dp(DType epsilon);
    long simp_curve_dp(DType epsilon, int, int, int * & );

    //output current hausdorff simplified chain with a required vertex number
    array<d3_point>& simp_curve_hausdorff(DType epsilon);

    double get_error() {return m_error; }

    int curve_len() {return m_size; }
    int curr_simp(){return m_vertex;} //return current simplified vertex number;

private:
    array<d3_point> m_original;
    array<d3_point> m_cur_curve;
    
    int    m_size; // original size
    int    m_vertex; // current vertex number
    DType  m_error; 

    // member functions
    void init(); // init all member data ....
    void reset(); // reset all member data .... 

    array<PointType>& getPi(int low, int high, array<PointType>& P);
    array<PointType>& getSeg(int low, int high, array<PointType>& Q); 

};

#endif
